==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/server/inc/xmlrequest.php <==
<?php
require_once "inputstream.php";
require_once "error.php";
/**
 * @brief	Common class for classes than implements 
			handling of part of XML request
 * Define interface for exception handling and getting answer for XML request
 */
abstract class XMLRequestInterface {
		public $oDoc;			  //DOM XML class with input xml	
		
		public $oQuery;			//DOM XML element representing actual query
		
		public $aAttrs;			//Attributes of an input / query
				 
		public $aData;							 // output data storage
		
		public $sTemplatePath = "inc/templates/";  // default template path
		
		public $sTemplateFile = 'default';		 //default template file

		public $sFormatType = 'xml';		 //default formatting for the response
		
		/**
		 *@brief : Answer method - it returns response XML for the client
		 */
		public function answer(){

			$sPath = ""; //template path
			
			$sNS = "";	//sub-queries xml namespace
		
			/* Compose template path */
			$sPath = $this->sTemplatePath.$this->sTemplateFile.templateext;

			/* Checkpath */
			if (!file_exists($sPath)){
        throw new Exc ('XMLRequestInterface answer: invalid template path ' . $sPath);
	    }
			/* Get the sub-querie's namespace type */
			$this->aData['xmlNameSpace'] = $this->aAttrs['namespace'];

			$tpltype = str_replace($this->sTemplatePath,'',$sPath);

			switch(strtolower($tpltype)){
        case 'webmailiqfriends_get.tpl':          //Osetreno vnitrne
        case 'webmailiqitems_get.tpl':            //Osetreno vnitrne
        case 'webmailiqgroups_get.tpl':           //Osetreno vnitrne
        case 'xmlrequest_iq_result.tpl':          //Osetreno vnitrne
        case 'webmailiqprivateandpublic_get.tpl': //Osetreno vnitrne
          break;
        default:
          //Encode chars to be used in XML

          if ($this->sFormatType=='xml')
            $this->aData = Tools::htmlspecialchars_array($this->aData);
        break;
      }

			$return =  trim(template($sPath,$this->aData));
		  return $return;
    }
		

		
		private function checkInputXML(){}
		
		private function exeInputXML(){}
}

/**
 * @brief	Class that represent XML request
 */
class XMLRequest extends XMLRequestInterface
{
		/* Permited XML roots */
		private $aXMLRootTags = Array(
				'iq' => 'WebmailIq',
		);  
		/*  Permited XML namespaces and their bindings to the proper object */
		private $aXMLNS = Array(
		    'webmail:iq:items' => 'Items',
		    'webmail:iq:folders' => 'Folders',
				'webmail:iq:accounts' => 'Accounts',
        'webmail:iq:private' => 'Private',
				'webmail:iq:public' => 'Public',
				'webmail:iq:message' => 'Message',
				'webmail:iq:freebusy' => 'FreeBusy',
				'webmail:iq:spellchecker' => 'SpellChecker',
				'webmail:iq:import' => 'Import',
				'webmail:iq:auth' => 'Auth'
		);
		/* Free XML namespaces (unauthorized access granted) */
		private $aFreeXMLNS = Array(
				'webmail:iq:auth' => 'Auth',
				'webmail:iq:public' => 'Public',
				'webmail:iq:freebusy' => 'FreeBusy',
				'webmail:iq:private' => 'Private',
		);
		
		private $aQuery = Array(); //Prefixes for XML namespaces
		/**
		* @brief	Public: Constructor
		* @param	request		String that contains XML request
		*/
		function __construct()
		{
		    
				//Init queries
				foreach($this->aXMLNS as $key => $val)
				  $this->aQuery[$key] = "/iq/".strtolower($val).":query";
	
				// Retirieve XML input
				$oInputStream = &new InputStream;
				
				/* TODO : Remove after DOM XML update */
				$this->sInputXML = $oInputStream->getString();

				// Check XML Input
				$this->checkInputXML($oInputStream->getDOM());
				
				/* TODO : Remove after DOM XML update */
				//$this->checkInputXML($oInputStream->getArray());

				// Execute XML
				if (!file_exists($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE)) 
          $this->exeInputXML();
        else {

          $this->aData['sid'] = $this->aAttrs['sid'];
          $this->aData['id'] = htmlspecialchars($this->aAttrs['id']);
          
          $this->sTemplateFile = 'webmailiqunavailable';
          echo $this->answer();
          die();
        }
		}

		/**
		* @brief	Private
		* Check if head of the request is valid
		* @param	$doc - DOM XML class with input XML document
		* @return  TRUE/FALSE, throws exception (when returns FALSE)
		*/
		private function checkInputXML($oDoc)
		{
				global $aError,$_iqdata,$sSTime;
				
        $iQueryCount = 0;	//Sub queries count
				$sNS		 = "";  //XML Namespace 

				/* Store IQ information */
				$this->sRootType = strtolower($oDoc->documentElement->tagName);
				$this->aAttrs["sid"] = $oDoc->documentElement->getAttribute('sid');
				$this->aAttrs["id"] = $oDoc->documentElement->getAttribute('uid');
				if ($format = $oDoc->documentElement->getAttribute('format'))
					$this->sFormatType = strtolower($format);
				else
				  $this->sFormatType = 'xml';
				  
				$sRequest = $oDoc->getNodeContent($oDoc->documentElement);
				
				
				
				$_iqdata['id'] = $this->aAttrs["id"];
        $_iqdata['format'] = $this->sFormatType;
        
				/* IQ check */
				if ($this->sRootType == 'iq'){

					// att: type
					$this->aAttrs["type"] = $oDoc->documentElement->getAttribute('type');
					
					/* IQ type check */
					switch($this->aAttrs["type"]){
					  case 'get': 
						  break;
						  
					  case 'set': 
						  break;
						  
					  case ''	: 
						  $this->aAttrs["type"]="get"; 
						  break;
						  
					  default	: 
						  throw new Exc ($aError['invalid type']);
					}

					// query att: xmlns
					foreach ($this->aQuery as $sQuery) {
					 foreach ($oDoc->query($sQuery) as $oQuery) {
            $iQueryCount++;

						/* Check XML namespace */ 
						if (!$sNS = $oQuery->namespaceURI) 
						  throw new Exc ('missing namespace attribute');
						 // $sNS = "webmail:iq:auth";
						/* Check user's session ID */
						if ( (!isset($this->aAttrs["sid"]) || $this->aAttrs["sid"]=="")
							&& !isset($this->aFreeXMLNS[$sNS]) )
						{
							throw new Exc ($aError['missing sid'],'session_mismatch');
						}
						$this->aAttrs["namespace"] = $sNS;
					 } 
          }
					
					/* Throws an exception if there are no query tags */
					if (!$iQueryCount){
           throw new Exc ($aError['missing query tag']);
				  }
        }
				
				/* Other root types (<message>) */
				elseif ((!isset($this->aAttrs["sid"]) || $this->aAttrs["sid"]=="")
						&&$this->sRootType=='message')
				{
					throw new Exc ($aError['missing sid']);
					
				}

				// checksid if is presented
				if ($this->aAttrs["sid"]!="" && isset($this->aAttrs["sid"]))
				{
				  User::load($this->aAttrs["sid"]);
				}
				/* Store input doc for executing */
				$this->oDoc = $oDoc;
				
        log_buffer($this->sInputXML, 'REQUEST', array('id' => $this->aAttrs["id"], 'sid' => $this->aAttrs["sid"]));
								
				return true;
		}

		/**
		* @brief	Protected: Do operations dependent on 'query xmlns'
		* @return  String that contains answer to XML request
		*/
		private function exeInputXML() {
		
				global $aError;
		
		    /* query att: xmlns */
				$this->aData['subAnswer'] = '';
				foreach ($this->aQuery as $sQuery) 
				  foreach($this->oDoc->query($sQuery) as $oQuery){
					  
					  /* Compose classname */
					  $sNS = $oQuery->namespaceURI;
					  $this->aAttrs['namespace'] = $sNS;
					  $classConstructor = $this->aXMLRootTags[$this->sRootType].$this->aXMLNS[$sNS];
					  
					  /* Include class file */
					  require_once "inc/".strtolower($classConstructor).".php";

					  /* Check for class existence */
					  if (!class_exists($classConstructor))
						 throw new Exc ($aError['class doesnt exists']." [$classConstructor]");

					  /* Create an subrequest and process an subanswer depending on XML namespace */
					  $cSubRequest = new $classConstructor($oQuery,$this->oDoc,$this->aAttrs);
					  $this->aData['subAnswer'] .= $cSubRequest->answer();
				}

				if ($this->sRootType=='message')
				{
					  require_once "inc/webmailiqmessage.php";
					  $cSubRequest = new WebmailIqMessage($this->oDoc->getNode("message"),$this->oDoc,$this->aAttrs);
					  $this->aData['subAnswer'] .= $cSubRequest->answer();
				
				}
				/* Remove an subAnswer key from result due template system */ 
				if(!$this->aData['subAnswer']) unset($this->aData['subAnswer']);
        
				/* Add session ID and IQ id to the result template*/ 
				$this->aData['sid'] = $this->aAttrs["sid"];
				$this->aData['id']  = htmlspecialchars($this->aAttrs["id"]);
				$this->aData['ns']  = $this->aAttrs["namespace"];

				$this->sTemplateFile= 'xmlrequest_'.$this->sRootType.'_result';
 		}
}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/server/inc/xmlrequest.php <==
<?php

require_once "inputstream.php";
require_once "error.php";
/**
 * @brief	Common class for classes than implements 
			handling of part of XML request
 * Define interface for exception handling and getting answer for XML request
 */
abstract class XMLRequestInterface {
		public $oDoc;			  //DOM XML class with input xml	
		
		public $oQuery;			//DOM XML element representing actual query
		
		public $aAttrs;			//Attributes of an input / query
				 
		public $aData;							 // output data storage
		
		public $sTemplatePath = "inc/templates/";  // default template path
		
		public $sTemplateFile = 'default';		 //default template file

		public $sFormatType = 'xml';		 //default formatting for the response
		
		// Free XML namespace actions 
		private static $aFreeXMLNSAction = Array(
				'webmail:iq:accounts' => array('signup'=>true)
		);
    		
		/**
		 *@brief : Answer method - it returns response XML for the client
		 */
		public function answer(){

			$sPath = ""; //template path
			
			$sNS = "";	//sub-queries xml namespace
		
			/* Compose template path */
			$sPath = $this->sTemplatePath.$this->sTemplateFile.templateext;

			/* Checkpath */
			if (!file_exists($sPath))
        throw new Exc('xmlrequest_path',$sPath);
	    
			/* Get the sub-querie's namespace type */
			$this->aData['xmlNameSpace'] = $this->aAttrs['namespace'];

			$tpltype = str_replace($this->sTemplatePath,'',$sPath);

			switch(strtolower($tpltype)){
        case 'webmailiqfriends_get.tpl':          //Osetreno vnitrne
        case 'webmailiqitems_get.tpl':            //Osetreno vnitrne
        case 'webmailiqgroups_get.tpl':           //Osetreno vnitrne
        case 'xmlrequest_iq_result.tpl':          //Osetreno vnitrne
        case 'webmailiqprivateandpublic_get.tpl': //Osetreno vnitrne
          break;
        default:
          //Encode chars to be used in XML

          if ($this->sFormatType=='xml')
            $this->aData = Tools::htmlspecialchars_array($this->aData);
        break;
      }

			$return =  trim(template($sPath,$this->aData));

		  return $return;
    }

 		
 		/*
 		 * Checks if the action in some namespace is free of providing SID(being logged)
 		 * @param $sNamespace : i.e. webmail:iq:items
 		 * @param $sAction : i.e. add
 		 */                   		 
 		public static function checkFreeAction($sNamespace,$sAction)
    {
     if(!isset(self::$aFreeXMLNSAction[$sNamespace][$sAction]))
      throw new Exc('xmlrequest_access_denied');
     return true;
    }
    
    		
		private function checkInputXML($oDoc){}
		
		private function exeInputXML(){}
}

/**
 * @brief	Class that represent XML request
 */
class XMLRequest extends XMLRequestInterface
{
		/* Permited XML roots */
		private $aXMLRootTags = Array(
				'iq' => 'WebmailIq',
		);  
		/*  Permited XML namespaces and their bindings to the proper object */
		private $aXMLNS = Array(
		    'webmail:iq:items' => 'Items',
		    'webmail:iq:folders' => 'Folders',
				'webmail:iq:accounts' => 'Accounts',
        'webmail:iq:private' => 'Private',
				'webmail:iq:public' => 'Public',
				'webmail:iq:message' => 'Message',
				'webmail:iq:freebusy' => 'FreeBusy',
				'webmail:iq:spellchecker' => 'SpellChecker',
				'webmail:iq:import' => 'Import',
				'webmail:iq:auth' => 'Auth'
		);
		/* Free XML namespaces (unauthorized access granted) */
		private $aFreeXMLNS = Array(
				'webmail:iq:auth' => 'Auth',
				'webmail:iq:accounts' => 'Accounts',
				'webmail:iq:public' => 'Public',
				'webmail:iq:freebusy' => 'FreeBusy',
				'webmail:iq:private' => 'Private',
		);
		

		private $aQuery = Array(); //Prefixes for XML namespaces
		/**
		* @brief	Public: Constructor
		* @param	request		String that contains XML request
		*/
		function __construct()
		{
		    
				//Init queries
				foreach($this->aXMLNS as $key => $val)
				  $this->aQuery[$key] = "/iq/".strtolower($val).":query";
	
				// Input stream object
				$oInputStream = new InputStream;
				
				/* XML input string */
				$this->sInputXML = $oInputStream->getString();

				// Check XML Input
				$this->checkInputXML($oInputStream->getDOM());

				// Execute XML
				if (!file_exists(@$_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE)) 
          $this->exeInputXML();
        else {

          $this->aData['sid'] = $this->aAttrs['sid'];
          $this->aData['id'] = htmlspecialchars($this->aAttrs['id']);
          
          $this->sTemplateFile = 'webmailiqunavailable';
          echo $this->answer();
          die();
        }
		}

		/**
		* @brief	Private
		* Check if head of the request is valid
		* @param	$doc - DOM XML class with input XML document
		* @return  TRUE/FALSE, throws exception (when returns FALSE)
		*/
		private function checkInputXML($oDoc)
		{
				global $aError,$_iqdata,$sSTime;
				
        $iQueryCount = 0;	//Sub queries count
				$sNS		 = "";  //XML Namespace 

				/* Store IQ information */
				$this->sRootType = strtolower($oDoc->documentElement->tagName);
				$this->aAttrs["sid"] = $oDoc->documentElement->getAttribute('sid');
				$this->aAttrs["id"] = $oDoc->documentElement->getAttribute('uid');
				if ($format = $oDoc->documentElement->getAttribute('format'))
					$this->sFormatType = strtolower($format);
				else
				  $this->sFormatType = 'xml';

				$sRequest = $oDoc->getNodeContent($oDoc->documentElement);

				$_iqdata['id'] = $this->aAttrs["id"];
        $_iqdata['format'] = $this->sFormatType;
        
				/* IQ check */
				if ($this->sRootType == 'iq'){

					// att: type
					$this->aAttrs["type"] = $oDoc->documentElement->getAttribute('type');
					
					/* IQ type check */
					switch($this->aAttrs["type"]){
					  case 'get': 
						  break;
						  
					  case 'set': 
						  break;
						  
					  case ''	: 
						  $this->aAttrs["type"]="get"; 
						  break;
						  
					  default	: 
						  throw new Exc('xmlrequest_invalid_type',$this->aAttrs["type"]);
					}

					// query att: xmlns
					foreach ($this->aQuery as $sQuery) {
					 foreach ($oDoc->query($sQuery) as $oQuery) {
            $iQueryCount++;

						/* Check XML namespace */ 
						if (!$sNS = $oQuery->namespaceURI) 
						  throw new Exc ('xmlrequest_missing_xmlns');
						 // $sNS = "webmail:iq:auth";
						/* Check user's session ID */
						if ( (!isset($this->aAttrs["sid"]) || $this->aAttrs["sid"]=="")
							&& !isset($this->aFreeXMLNS[$sNS]) )
						{
							throw new Exc ('xmlrequest_invalid_sid');
						}
						$this->aAttrs["namespace"] = $sNS;
					 } 
          }

					/* Throws an exception if there are no query tags */
					if (!$iQueryCount){
           throw new Exc ('xmlrequest_missing_tag','query');
				  }
        }
				
				/* Other root types (<message>) */
				elseif ((!isset($this->aAttrs["sid"]) || $this->aAttrs["sid"]=="")
						&&$this->sRootType=='message')
				{
					throw new Exc ('xmlrequest_invalid_sid');
					
				}

				// checksid if is presented
				if ($this->aAttrs["sid"]!="" && isset($this->aAttrs["sid"]))
				{
				  User::load($this->aAttrs["sid"]);
				}
				/* Store input doc for executing */
				$this->oDoc = $oDoc;
				
        log_buffer($this->sInputXML, 'REQUEST', array('id' => $this->aAttrs["id"], 'sid' => $this->aAttrs["sid"]));
								
				return true;
		}
		
		
    /**
		* @brief	Protected: Do operations dependent on 'query xmlns'
		* @return  String that contains answer to XML request
		*/
		private function exeInputXML() {
		
				global $aError;
		
		    /* query att: xmlns */
				$this->aData['subAnswer'] = '';

				foreach ($this->aQuery as $sQuery) 
				  foreach($this->oDoc->query($sQuery) as $oQuery){
					  
					  /* Compose classname */
					  $sNS = $oQuery->namespaceURI;
					  $this->aAttrs['namespace'] = $sNS;
					  $classConstructor = $this->aXMLRootTags[$this->sRootType].$this->aXMLNS[$sNS];

					  /* Include class file */ 
					  require_once "inc/".strtolower($classConstructor).".php";

					  /* Check for class existence */
					  if (!class_exists($classConstructor))
						 throw new Exc ('xmlrequest_missing_class',$classConstructor);

					  /* Create an subrequest and process an subanswer depending on XML namespace */
					  $cSubRequest = new $classConstructor($oQuery,$this->oDoc,$this->aAttrs);

					  $this->aData['subAnswer'] .= $cSubRequest->answer();
				}

				if ($this->sRootType=='message')
				{
					  require_once "inc/webmailiqmessage.php";
					  $cSubRequest = new WebmailIqMessage($this->oDoc->getNode("message"),$this->oDoc,$this->aAttrs);
					  $this->aData['subAnswer'] .= $cSubRequest->answer();
				
				}
				/* Remove an subAnswer key from result due template system */ 
				if(!$this->aData['subAnswer']) unset($this->aData['subAnswer']);
        
				/* Add session ID and IQ id to the result template*/ 
				$this->aData['sid'] = $this->aAttrs["sid"];
				$this->aData['id']  = htmlspecialchars($this->aAttrs["id"]);
				$this->aData['ns']  = $this->aAttrs["namespace"];

				$this->sTemplateFile= 'xmlrequest_'.$this->sRootType.'_result';
 		}
		

}
?>
